function getLocalStorage() {
  try {
    return localStorage;
  } catch (err) {
    return null;
  }
}

function getSessionStorage() {
  try {
    return sessionStorage;
  } catch (err) {
    return null;
  }
}

function validateCustomStorage(storage) {
  if (typeof storage.getItem !== 'function' || typeof storage.setItem !== 'function') throw new Error('Persistence: Invalid storage.');
  return storage;
}

function getItem(storage, key, deserialize, callback) {
  const res = storage.getItem(key);
  if (res === null) return;
  if (typeof res !== 'string' && typeof res.then === 'function') res.then(val => {
    if (val != null) processValue(deserialize, val, callback);
  });else processValue(deserialize, res, callback);
}

function setItem(storage, key, value, serialize) {
  processValue(serialize, value, res => storage.setItem(key, res));
}

function processValue(func, value, callback) {
  const res = func(value);
  if (res && typeof res.then === 'function') res.then(callback);else callback(res);
}

export default function persistence(key, options = {}) {
  if (typeof key !== 'string') throw new Error('Persistence requires a string key.');
  let storage;
  const storageOption = options.storage ?? 'local';
  if (storageOption === 'local') storage = getLocalStorage();else if (storageOption === 'session') storage = getSessionStorage();else storage = validateCustomStorage(storageOption);

  if (!storage) {
    console.warn('Storage unavailable. Persistence disabled.');
    return {};
  }

  return {
    init(origInit, entity) {
      return () => {
        const deserialize = options.deserializeFn || JSON.parse;
        origInit();
        getItem(storage, key, deserialize, entity.set);
      };
    },

    set(origSet, entity) {
      return (...args) => {
        const serialize = options.serializeFn || JSON.stringify;
        origSet(...args);
        setItem(storage, key, entity.get(), serialize);
      };
    }

  };
}