import _extends from "@babel/runtime/helpers/esm/extends";
import { getMutableMap, initRegistry, updateRegistry, watchRegistry } from './registry';
let devTools = null;
let isInspectorEnabled = false;
let isInspectorInitialized = false;
let isDevToolsInitialized = false;
let isDevToolsPaused = false;
let initialRegistryValue = null;
export function enableInspector(condition = true) {
  isInspectorEnabled = condition;
}
export const features = {
  pause: true,
  export: true,
  import: 'custom',
  jump: true,
  skip: true,
  reorder: true
};
export function initInspector() {
  var _window$__REDUX_DEVTO;

  devTools = ((_window$__REDUX_DEVTO = window.__REDUX_DEVTOOLS_EXTENSION__) == null ? void 0 : _window$__REDUX_DEVTO.connect({
    name: document.title,
    features
  })) ?? null;

  if (devTools) {
    initRegistry();
    devTools.subscribe(handleDevToolsEvent);
  }

  isInspectorInitialized = true;
}

function handleDevToolsEvent(event) {
  if (!isInspectorEnabled || !devTools) return;

  if (event.type === 'DISPATCH') {
    switch (event.payload.type) {
      case 'JUMP_TO_STATE':
      case 'JUMP_TO_ACTION':
        applyStateToEntities(event.state);
        break;

      case 'COMMIT':
        devTools.init(getMutableMap());
        break;

      case 'ROLLBACK':
        const snapshot = applyStateToEntities(event.state);

        if (snapshot) {
          devTools.init(snapshot);
        }

        break;

      case 'RESET':
        if (initialRegistryValue !== null) {
          updateRegistry(initialRegistryValue);
          devTools.init(initialRegistryValue);
        }

        break;

      case 'IMPORT_STATE':
        {
          var _imported$computedSta;

          const {
            nextLiftedState: imported
          } = event.payload;
          if (!imported || !imported.computedStates) return;
          const currentIndex = imported.currentStateIndex ?? imported.computedStates.length - 1;
          const currentState = (_imported$computedSta = imported.computedStates[currentIndex]) == null ? void 0 : _imported$computedSta.state;
          if (typeof currentState !== 'object') return;
          updateRegistry(currentState);
          devTools.send(null, imported);
          return;
        }

      case 'PAUSE_RECORDING':
        isDevToolsPaused = !isDevToolsPaused;
        break;
    }
  }
}

function applyStateToEntities(state) {
  let parsedState;

  try {
    parsedState = JSON.parse(state);
  } catch (err) {
    console.error('Ignoring invalid state received from DevTools.');
  }

  if (typeof parsedState !== 'object') return;
  updateRegistry(parsedState);
  return parsedState;
}

export function onInit(entity) {
  if (entity.name.charAt(0) === '_') return;
  if (!isInspectorInitialized) initInspector();
  if (!devTools) return;
  const mutableMap = getMutableMap();

  if (!(entity.name in mutableMap)) {
    watchRegistry(value => {
      if (!isInspectorEnabled) return;
      entity.set(value[entity.name], '@@DEVTOOLS');
    });
  }

  mutableMap[entity.name] = entity.get();

  if (isDevToolsInitialized && isInspectorEnabled && !isDevToolsPaused) {
    devTools.send({
      type: `${entity.name}:@@LAZY_INIT`
    }, mutableMap);
  }
}
export function onSet(entity, alias) {
  if (entity.name.charAt(0) === '_') return;
  if (!devTools) return;
  const mutableMap = getMutableMap();
  mutableMap[entity.name] = entity.get();
  if (!isInspectorEnabled) return;

  if (!isDevToolsInitialized) {
    devTools.init(mutableMap);
    isDevToolsInitialized = true;
    initialRegistryValue = _extends({}, mutableMap);
  }

  if (!isDevToolsPaused) {
    devTools.send({
      type: `${entity.name}:${alias}`
    }, mutableMap);
  }
}