import type { MutableMap } from './registry';
import type { Entity } from '../core/entity';
declare type DevToolsEvent = {
    type: 'DISPATCH';
    payload: {
        type: string;
        nextLiftedState?: {
            computedStates: {
                state: any;
            }[];
            currentStateIndex?: number;
        };
        [P: string]: any;
    };
    state: string;
} | {
    type: 'ACTION';
    payload: string;
};
interface DevToolsConnection {
    init(state: MutableMap): void;
    send(action: {
        type: string;
    } | null, state: MutableMap): void;
    subscribe(callback: (event: DevToolsEvent) => void): void;
}
declare global {
    interface Window {
        __REDUX_DEVTOOLS_EXTENSION__?: {
            connect(options: {
                name?: string;
                features?: Record<string, any>;
            }): DevToolsConnection;
        };
    }
}
/**
 * Enables the Inspector for visual debugging of entities using the
 * Redux Dev Tools browser extension.
 *
 * This function must be invoked at the topmost level, normally the React
 * app's `index` file. The optional `condition` typically checks if the
 * app is running in Production mode, in which case you would usually want
 * Inspector to be disabled.
 *
 * @see https://github.com/reduxjs/redux-devtools
 *
 * @param condition - should be `true` to enable Inspector
 */
export declare function enableInspector(condition?: boolean): void;
export declare const features: {
    pause: boolean;
    export: boolean;
    import: string;
    jump: boolean;
    skip: boolean;
    reorder: boolean;
};
export declare function initInspector(): void;
export declare function onInit(entity: Entity): void;
export declare function onSet(entity: Entity, alias: string): void;
export {};
